// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/IdentifiedFinalState.hh"
#include "Rivet/Projections/ZFinder.hh"
#include "Rivet/Projections/VetoedFinalState.hh"
#include "Rivet/Tools/ParticleIdUtils.hh"
#include "Rivet/Projections/MergedFinalState.hh"
#include "Rivet/ParticleName.hh"

namespace Rivet {
  class CMS_2012_I1298807 : public Analysis {
  public:
    // Constructor
    CMS_2012_I1298807() : Analysis("CMS_2012_I1298807") {
      leading_l_pt = 0.;
      second_l_pt = 0.;
    }

    void init() {
      FinalState fs(Cuts::abseta < 2.5);
      IdentifiedFinalState leptons(fs);
      leptons.acceptChLeptons();
      addProjection(leptons, "leptons");

      IdentifiedFinalState electrons(Cuts::abseta < 2.5);
      electrons.acceptIdPair(PID::ELECTRON);
      IdentifiedFinalState muons(Cuts::abseta < 2.4);
      muons.acceptIdPair(PID::MUON);

      MergedFinalState leptons_cuts(electrons, muons);
      addProjection(electrons, "ELECTRONS");
      addProjection(muons, "MUONS");
      addProjection(leptons_cuts, "LEPTONS_CUTS");

      Cut cut_el = (Cuts::abseta < 2.5) && (Cuts::pT >= 7.0*GeV);
      Cut cut_mu = (Cuts::abseta < 2.4) && (Cuts::pT >= 5.0*GeV);

      ZFinder zeefinder(FinalState(), cut_el, PID::ELECTRON, 60*GeV,
                        120*GeV, 0.1, ZFinder::CLUSTERNODECAY, ZFinder::TRACK);
      addProjection(zeefinder, "ZeeFinder");

      ZFinder zmmfinder(FinalState(), cut_mu, PID::MUON, 60*GeV,
                        120*GeV, 0.1, ZFinder::CLUSTERNODECAY, ZFinder::TRACK);
      addProjection(zmmfinder, "ZmmFinder");

      VetoedFinalState fs_woZmm;
      fs_woZmm.addVetoOnThisFinalState(zmmfinder);

      VetoedFinalState fs_woZee;
      fs_woZee.addVetoOnThisFinalState(zeefinder);
      ZFinder zmmfinder_woZmm(fs_woZmm, cut_mu, PID::MUON, 60*GeV,
                              120*GeV, 0.1, ZFinder::CLUSTERNODECAY);
      addProjection(zmmfinder_woZmm, "Zmmfinder_WoZmm");
      ZFinder zeefinder_woZee(fs_woZee, cut_el, PID::ELECTRON, 60*GeV,
                              120*GeV, 0.1, ZFinder::CLUSTERNODECAY);
      addProjection(zeefinder_woZee, "Zeefinder_WoZee");

      _hist_pt_l1 = bookHisto1D(1, 1, 1);
      _hist_pt_z1 = bookHisto1D(1, 1, 2);
      _hist_pt_zz = bookHisto1D(1, 1, 3);
      _hist_m_zz = bookHisto1D(1, 1, 4);
      _hist_dphi_zz = bookHisto1D(1, 1, 5);
      _hist_dR_zz = bookHisto1D(1, 1, 6);

    }

    // Perform the per-event analysis
    void analyze(const Event& evt) {
      const double weight = evt.weight();

      const ParticleVector& leptons =
        applyProjection<IdentifiedFinalState>(evt,"leptons").particlesByPt();
      const FinalState& allleptonsfs =
        applyProjection<FinalState>(evt, "LEPTONS_CUTS");
      ParticleVector allleptons = allleptonsfs.particlesByPt();

      if (allleptons.size() >= 1) {
        double ptlept = allleptons[0].momentum().pT()/GeV;
        leading_l_pt = ptlept;
        if (allleptons.size() >= 2) {
          double ptlept2 = allleptons[1].momentum().pT()/GeV;
          second_l_pt = ptlept2;
        }
      }

      if (!(leading_l_pt > 20. && second_l_pt > 10.)) vetoEvent;

      const ZFinder& zeefinder = applyProjection<ZFinder>(evt, "ZeeFinder");
      const ZFinder& zmmfinder = applyProjection<ZFinder>(evt, "ZmmFinder");
      const ZFinder& zmmfinder_woZmm =
        applyProjection<ZFinder>(evt, "Zmmfinder_WoZmm");
      const ZFinder& zeefinder_woZee =
        applyProjection<ZFinder>(evt, "Zeefinder_WoZee");

      if ((zeefinder.bosons().size() > 0 &&
           zmmfinder.bosons().size() > 0) ||
          (zeefinder.bosons().size() > 0 &&
           zeefinder_woZee.bosons().size() > 0) ||
          (zmmfinder.bosons().size() > 0 &&
           zmmfinder_woZmm.bosons().size() > 0)) {

        // Zs ordered randomly
        FourMomentum pZ_a ;
        FourMomentum pZ_b ;

        // Zs ordered by pT
        FourMomentum pZ_1;
        FourMomentum pZ_2;

        FourMomentum pZZ ;
        FourMomentum Z_a_l1;
        FourMomentum Z_a_l2;
        FourMomentum Z_b_l1;
        FourMomentum Z_b_l2;


        if (zeefinder.bosons().size() > 0 &&
            zmmfinder.bosons().size() > 0) {
          pZ_a = zeefinder.bosons()[0].momentum();
          pZ_b = zmmfinder.bosons()[0].momentum();
          pZZ = pZ_a + pZ_b;

          pZ_1 = pZ_a;
          pZ_2 = pZ_b;

          Z_a_l1 = zeefinder.constituents()[0].momentum();
          Z_a_l2 = zeefinder.constituents()[1].momentum();
          Z_b_l1 = zmmfinder.constituents()[0].momentum();
          Z_b_l2 = zmmfinder.constituents()[1].momentum();
        }

        if (zeefinder.bosons().size() > 0 &&
            zeefinder_woZee.bosons().size() > 0) {
          pZ_a = zeefinder.bosons()[0].momentum();
          pZ_b = zeefinder_woZee.bosons()[0].momentum();
          pZZ = pZ_a + pZ_b;

          pZ_1 = pZ_a;
          pZ_2 = pZ_b;

          Z_a_l1 = zeefinder.constituents()[0].momentum();
          Z_a_l2 = zeefinder.constituents()[1].momentum();
          Z_b_l1 = zeefinder_woZee.constituents()[0].momentum();
          Z_b_l2 = zeefinder_woZee.constituents()[1].momentum();
        }

        if (zmmfinder.bosons().size() > 0 &&
            zmmfinder_woZmm.bosons().size() > 0) {
          pZ_a = zmmfinder.bosons()[0].momentum();
          pZ_b = zmmfinder_woZmm.bosons()[0].momentum();
          pZZ = pZ_a + pZ_b;
          pZ_1 = pZ_a;
          pZ_2 = pZ_b;

          Z_a_l1 = zmmfinder.constituents()[0].momentum();
          Z_a_l2 = zmmfinder.constituents()[1].momentum();
          Z_b_l1 = zmmfinder_woZmm.constituents()[0].momentum();
          Z_b_l2 = zmmfinder_woZmm.constituents()[1].momentum();
        }

        double pt_l1 = Z_a_l1.pT()/GeV;
        if (Z_a_l2.pT()/GeV > pt_l1) pt_l1 = Z_a_l2.pT()/GeV;
        if (Z_b_l1.pT()/GeV > pt_l1) pt_l1 = Z_b_l1.pT()/GeV;
        if (Z_b_l2.pT()/GeV > pt_l1) pt_l1 = Z_b_l2.pT()/GeV;

        // leading Z pt:
        double pt_z1 = pZ_a.pT() / GeV;
        if (pZ_b.pT() > pZ_a.pT()) {
          pt_z1 = pZ_b.pT() / GeV;
          pZ_1 = pZ_b;
          pZ_2 = pZ_a;
        }

        _hist_pt_zz->fill(pZZ.pT() / GeV, weight);
        _hist_m_zz->fill(pZZ.mass() / GeV, weight);
        _hist_dphi_zz->fill(deltaPhi(pZ_a, pZ_b), weight);
        _hist_dR_zz->fill(deltaR(pZ_a, pZ_b, PSEUDORAPIDITY), weight);
        _hist_pt_z1->fill(pt_z1, weight);
        _hist_pt_l1->fill(pt_l1, weight);
      } // end if ZZ
    }

    // Scale histograms
    void finalize() {
      double sum_height_pt_zz = 0;
      for (size_t i = 0; i < _hist_pt_zz->numBins(); i++) {
        _hist_pt_zz->bin(i).scaleW(1. / _hist_pt_zz->bin(i).width());
        sum_height_pt_zz += _hist_pt_zz->bin(i).height();
      }
      scale(_hist_pt_zz, 1. / sum_height_pt_zz);

      double sum_height_m_zz = 0;
      for (size_t i = 0; i < _hist_m_zz->numBins(); i++) {
        _hist_m_zz->bin(i).scaleW(1. / _hist_m_zz->bin(i).width());
        sum_height_m_zz += _hist_m_zz->bin(i).height();
      }
      scale(_hist_m_zz, 1. / sum_height_m_zz);

      double sum_height_dphi_zz = 0;
      for (size_t i = 0; i < _hist_dphi_zz->numBins(); i++) {
        _hist_dphi_zz->bin(i).scaleW(1. / _hist_dphi_zz->bin(i).width());
        sum_height_dphi_zz += _hist_dphi_zz->bin(i).height();
      }
      scale(_hist_dphi_zz, 1. / sum_height_dphi_zz);

      double sum_height_dR_zz = 0;
      for (size_t i = 0; i < _hist_dR_zz->numBins(); i++) {
        _hist_dR_zz->bin(i).scaleW(1. / _hist_dR_zz->bin(i).width());
        sum_height_dR_zz += _hist_dR_zz->bin(i).height();
      }
      scale(_hist_dR_zz, 1. / sum_height_dR_zz);


      double sum_height_pt_z1 = 0;
      for(size_t i = 0; i < _hist_pt_z1->numBins(); i++) {
        _hist_pt_z1->bin(i).scaleW(1. / _hist_pt_z1->bin(i).width());
        sum_height_pt_z1 += _hist_pt_z1->bin(i).height();
      }
      scale(_hist_pt_z1, 1. / sum_height_pt_z1);

      double sum_height_pt_l1 = 0;
      for(size_t i = 0; i < _hist_pt_l1->numBins(); i++) {
        _hist_pt_l1->bin(i).scaleW(1. / _hist_pt_l1->bin(i).width());
        sum_height_pt_l1 += _hist_pt_l1->bin(i).height();
      }
      scale(_hist_pt_l1, 1. / sum_height_pt_l1);
    }

  private:
    // Data members like post-cuts event weight counters go here
    double leading_l_pt;
    double second_l_pt;

    Histo1DPtr _hist_pt_zz;
    Histo1DPtr _hist_m_zz;
    Histo1DPtr _hist_dphi_zz;
    Histo1DPtr _hist_dR_zz;

    Histo1DPtr _hist_pt_z1;
    Histo1DPtr _hist_pt_l1;
  };

  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(CMS_2012_I1298807);
}
